"""
GUI Model layer for REST Tester application.
Contains data models and GUI widgets for server and client instances.
"""

from .model import ConfigModel, ServerInstance, ClientInstance
from .server_instance_gui import ServerInstanceWidget
from .client_instance_gui import ClientInstanceWidget

__all__ = [
    'ConfigModel', 
    'ServerInstance', 
    'ClientInstance',
    'ServerInstanceWidget',
    'ClientInstanceWidget'
]
